package com.azt.itower.game;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.*;
import com.azt.itower.util.*;

/**
 *
 * Objects of the class card represent the graphical cards of the game
 * Objects of the Card type can paint themselves and can detect if they were hit.
 * @author thanassis@thanassis.com
 * @version 0.9 beta
 */
public class Card {
    private static Font titleFont = new Font("Helvetica", Font.BOLD, 12);
    private static Font messageFont = new Font("Helvetica", Font.BOLD, 12);
    private static Font costFont = new Font("Helvetica", Font.BOLD, 12);
    private Point origin;
    private Point referenceToGraphic;
    private Dimension size;
    private Image fImage;
    private CardModel model;
    private Image fCroppedCardImage;
    private Image fShadowedCardImage;
    private boolean isDiscarded = false;
    private boolean isSelected = false;
    private boolean isCropped = false;
    private boolean isExecutable = true;
    private boolean hasBeenExecuted = false;
    private int fWeight = 0;


    /**
     * The constructor of a card object needs a cardmodel that provides
     * all the behaviour, an image -that usually is the whole deck-
     * and coordinates to point out the start of the card in the image
     * @param CardModel
     * @param Image
     * @param Point
     *
     */
    public Card(CardModel model, Image image, Point p) {
        origin = new Point(0, 0);
        size = new Dimension(96, 126);
        referenceToGraphic = p;
        fImage = image;
        this.model = model;


    }

    /**
     * Getter for the CardModel that this card contains
     * @return CardModel The CardModel that this card contains
     */
    public CardModel getModel() {
      return model;
    }


    /**
     * returns the cost of the card
     * delegates the call to the model
     */
    public int getCost() {
        return model.getCost();
    }

    /**
     * executes the card
     * delegates the call to the model
     */
    public void execute() {
        model.execute();
        hasBeenExecuted=true;
    }

    /**
     * tests if the card  can be executed
     * delegates the call to the model
     * @return boolean
     */
    public boolean canExecute() {
        return model.canExecute();
    }

    /**
     * returns the name of a card
     * delegates the call to the model
     * @return String
     */
     public String getName() {
        return model.getName();
    }


    /**
     * sets the discarded state
     * when a card is discarded it can be painted differently
     */
    public void setDiscarded(boolean b) {
        hasBeenExecuted = !b;
        isDiscarded = b;
    }

    /**
     * Queries about the discarded state
     */
    private boolean isDiscarded() {
        return isDiscarded;
    }

    /**
     * Querries about the discarded state
     */
    public boolean isSelected() {
        return isSelected;
    }

    /**
     * Resets the states of the card
     */
    public void reset() {
        isSelected = false;
        isDiscarded = false;
    }

    /**
     * Crops the card
     */
    public void cropCard(Graphics g, Panel panel) {
        int x = origin.x;
        int y = origin.y;
        //int dy = 0;
       // if (referenceToGraphic.y == 2) dy = 2;
        fCroppedCardImage = panel.createImage(
            new FilteredImageSource(fImage.getSource(),
            new CropImageFilter(referenceToGraphic.x * size.width, referenceToGraphic.y * size.height,
            size.width, size.height)));
    }

    /**
     * paints the card to a panel
     * takes into consideration the state of the card
     */
    public void paint(Graphics g, Panel panel) {
        isSelected = false;
        if (!isCropped) {
            cropCard(g, panel);
            float transp = 0.6f;
            if (!IvoryTower.runAsApplication)
               transp = 0.3f;

            fShadowedCardImage = panel.createImage(
                new FilteredImageSource(fCroppedCardImage.getSource(),
                new GrayImageFilter()));

            fShadowedCardImage = panel.createImage(
                new FilteredImageSource(fCroppedCardImage.getSource(),
                new TransparentImageFilter(transp)));
            MediaTracker mt = new MediaTracker(panel);
            mt.addImage(fShadowedCardImage, 0);
            try {
                mt.waitForAll();
                isCropped = true;
            } catch (Exception ignore) {
            }
            //TransparentImageFilter(.9f)));
        }
        paintOpenCard(g, panel);

       if (!canExecute()) {
            paintShadow(g, panel);
       }

        /*
        if (isDiscarded()) {
            paintDiscardedView(g, panel);
        }
        */
    }

    /**
     * sets the origin of the card
     */
    public void setOrigin(Point p) {
        origin = p;
    }

    /**
     * paints a shadow on a card that can be played
     */
    private void paintShadow(Graphics g, Panel panel) {
        int x = origin.x;
        int y = origin.y;
        g.fillRect(x, y, size.width, size.height);
        g.drawImage(fShadowedCardImage, x, y, panel);

        g.setColor(Color.red);
        g.drawRect(x, y, size.width, size.height);
        x++; y++;
        /* Emboss Effect on the titles?
        g.setColor(Color.black);
        g.setFont(titleFont);
        g.drawString(model.getName(), x + 9, y + 14);
        g.setFont(messageFont);
        drawActionTitle(g,model.getActionTitle(), x + 6, y + 91);
        //g.drawString(model.getActionTitle(), x + 6, y + 91);
        g.setFont(costFont);
        g.drawString("" + model.getCost(), x + 78, y + 116);
        g.drawString("CAN'T PLAY", x + 18, y + 66);
        */

        g.setColor(Color.black);
        g.setFont(titleFont);
        g.drawString(model.getName(), x + 8, y + 13);
        g.setFont(messageFont);
        drawActionTitle(g,model.getActionTitle(), x + 5, y + 90);
        //g.drawString(model.getActionTitle(), x + 5, y + 90);
        g.setFont(costFont);

        g.drawString("" + model.getCost(), x + 77, y + 115);
        g.setColor(Color.red);
        g.drawString("CAN'T PLAY", x + 17, y + 65);

    }


    /**
     * paints the discarded card view
     */
    private void paintDiscardedView(Graphics g, Panel panel) {
        int x = origin.x;
        int y = origin.y;
        g.setColor(Color.blue);
        g.drawRect(x + 10, y + 10, size.width - 20, size.height - 20);
        g.drawRect(x + 15, y + 15, size.width - 30, size.height - 30);
        g.drawRect(x + 20, y + 20, size.width - 40, size.height - 40);
        g.drawString("DISCARDED", x + 30, y + 30);
    }

    /**
     * hook to paint the card on the deck
     */
    public void paintAsDeck(Graphics g, Panel panel) {
//        Image imageBuf = panel.createImage(80, 120);
//        Graphics Grbuffer = imageBuf.getGraphics();
       if (!isCropped) {
            cropCard(g, panel);
            float transp = 0.6f;
            if (!IvoryTower.runAsApplication)
               transp = 0.4f;

            fShadowedCardImage = panel.createImage(
                new FilteredImageSource(fCroppedCardImage.getSource(),
                new GrayImageFilter()));
            fShadowedCardImage = panel.createImage(
                new FilteredImageSource(fCroppedCardImage.getSource(),
                new TransparentImageFilter(transp)));
            MediaTracker mt = new MediaTracker(panel);
            mt.addImage(fShadowedCardImage, 0);
            try {
                mt.waitForAll();
                isCropped = true;
            } catch (Exception ignore) {

            }

        }
        if (isCropped) {
        paintOpenCard(g, panel);
         if (!hasBeenExecuted) {
          paintShadow(g, panel);
         }
        }
//       g.drawImage(imageBuf, 0, 0, panel);
    }

    /**
     * paints a closed card
     */
    private void paintClosedCard(Graphics g, Panel panel) {
        int x = origin.x;
        int y = origin.y;
        g.setColor(Color.red);
        g.fillRect(x, y, size.width, size.height);
    }

    /**
     * paints the selection rectangle
     */
    public void paintSelection(Graphics g, Color c) {
        isSelected = true;
        int x = origin.x;
        int y = origin.y;
        g.setColor(c);
        g.drawRect(x, y, size.width, size.height);
        g.drawRect(x + 1, y + 1, size.width - 2, size.height - 2);
        g.drawRect(x + 2, y + 2, size.width - 4, size.height - 4);
        g.drawRect(x + 3, y + 3, size.width - 6, size.height - 6);
    }

    /**
     * paints card normally
     */
    private void paintOpenCard(Graphics g, Panel panel) {
        int x = origin.x;
        int y = origin.y;
        g.setColor(Color.black);
        g.fillRect(x, y, size.width, size.height);
        g.setColor(Color.white);

      /* This bit of code had to go as BufferedImage is not supported in jdk1.1
      BufferedImage bi = new BufferedImage(fImage.getWidth(panel),fImage.getHeight(panel), BufferedImage.TYPE_INT_RGB) ;
      BufferedImage cbi = new BufferedImage(size.width,size.height, BufferedImage.TYPE_INT_RGB) ;

      Graphics tempGraphics = bi.createGraphics();
      Graphics cbg = cbi.createGraphics();

      int dy = 0 ;
      if (referenceToGraphic.y==2) dy=2;

      tempGraphics.drawImage(fImage, -referenceToGraphic.x*size.width, -(referenceToGraphic.y*size.height)-dy, panel);
      cbg.drawImage(bi,0,0,panel);
      g.drawImage(cbi,x,y,panel);
*/

      /* //Double buffering?
      BufferedImage bi = new BufferedImage(size.width,size.height, BufferedImage.TYPE_INT_RGB);
      Graphics tempGraphics = bi.createGraphics();
      tempGraphics.drawImage(fCroppedCardImage, 0,0,panel);
      */
      g.drawImage(fCroppedCardImage, x, y, panel);
      g.drawRect(x, y, size.width, size.height);
/*
      g.setFont(titleFont);

      g.setColor(Color.black);
      g.drawString(model.getName(), x + 9, y + 14);
      g.setFont(messageFont);
      g.drawString(model.getActionTitle(), x + 6, y + 91);
      g.setFont(costFont);
      g.drawString("" + model.getCost(), x + 78, y + 116);
*/
      g.setColor(Color.black);
      g.setFont(titleFont);
      x++;
      y++;
      g.drawString(model.getName(), x + 8, y + 13);
      g.setFont(messageFont);

      //g.drawString(model.getActionTitle(), x + 5, y + 90);
      drawActionTitle(g,model.getActionTitle(), x + 5, y + 90);
      g.setFont(costFont);
      g.drawString("" + model.getCost(), x + 77, y + 115);

    }


    private void drawActionTitle(Graphics g, String title,int x, int y) {
       StringTokenizer stz = new StringTokenizer(title);
       int counter=0;
       for (int i = 0; i < 3; i++) {
         String st1=null, st2=null;
         String line = "";
         if (stz.hasMoreTokens()) st1 = stz.nextToken();
         if (stz.hasMoreTokens()) st2 = stz.nextToken();
         if (st1!=null) line = st1;
         if (st2!=null) line = line +" " + st2;
         g.drawString(line,x+2,y+14*i);
       }

    }

    /**
     * finds out if a coordinate belongs to the rectangle surface of the card
     * @return boolean
     */
    public boolean isHit(int x, int y) {
        return ((x > origin.x) && (x < origin.x + size.width) && (y > origin.y) && (y < origin.y + size.height));
    }

    public int getWeight() {
      return fWeight;
    }

    public void setWeight(int w) {
      fWeight = w;
    }
}
